//Brent Cowan - 2011

#pragma comment(lib, "Gadget/DevIL/DevIL.lib")
#pragma comment(lib, "Gadget/DevIL/ILU.lib")
#pragma comment(lib, "Gadget/DevIl/ILUT.lib")

#include "DevIL/il.h"
#include "DevIL/ilu.h"
#include "DevIL/ilut.h"

#include "GLFW/IncludesGLFW.h"
#include "Extensions.h"
#include "Texture.h" 
#include "Image.h"

bool Image::init = false;

Image::Image()
{
	width = 0; height = 0; GL_ID = 0; 
	keepData = false;
	clamp = false;
	data = NULL;

	//Initialize DevIL if it hasn't been. 
	//Note: it is not a problem if this is called more than once
	if(!init)
	{
		ilInit(); iluInit();
		ilutRenderer(ILUT_OPENGL);
		init = true;
	}
}

Image::~Image()
{
	if(glIsTexture(GL_ID))
	{
		glDeleteTextures(1, &GL_ID);
	}
	
	if(data!=NULL){delete [] data; data = NULL;}
}

bool Image::Load(const char *FileName)
{
	if(data!=NULL){delete [] data; data = NULL;}

	//Delete the texture from OpenGL if it exists
	if(glIsTexture(GL_ID))
	{
		glDeleteTextures(1, &GL_ID);
	}

	//Load the image
	ILuint DevIL_ID;
    ilGenImages(1, &DevIL_ID);
    ilBindImage(DevIL_ID);

    if (!ilLoadImage(FileName)) 
	{
        ilDeleteImages(1, &DevIL_ID);
        return false;
    }

    //Convert paletted images to RGB format
    if (ilGetInteger(IL_IMAGE_FORMAT) == IL_COLOR_INDEX)
    {
        if (!ilConvertImage(IL_RGB, IL_UNSIGNED_BYTE))
        {
            ilDeleteImages(1, &DevIL_ID);
            return false;
        }
    }

 	ilBindImage(DevIL_ID);

    //Get image information
    components = (GLenum) ilGetInteger(IL_IMAGE_BYTES_PER_PIXEL);
	unsigned int type       = (GLenum) ilGetInteger(IL_IMAGE_TYPE);
    unsigned int format     = (GLenum) ilGetInteger(IL_IMAGE_FORMAT);
    
	ilBindImage(DevIL_ID);
	ilActiveMipmap(0);
	width  = ilGetInteger(IL_IMAGE_WIDTH);
    height = ilGetInteger(IL_IMAGE_HEIGHT);

	data = new GLubyte[components*width*height];
	const GLubyte* imageData   = (GLubyte*) ilGetData();

	for(int w=0; w<width*components; w++){for(int h=0; h<height; h++)
	{
		data[w+(height-h-1)*width*components] = imageData[w+h*width*components];
	}}//End triple for loop

	//Delete temporary IL image data
	ilDeleteImages(1, &DevIL_ID);

	glGenTextures(1, &GL_ID);
    glBindTexture(GL_TEXTURE_2D, GL_ID);

	const GLvoid* curData   = (GLvoid*) data;

	gluBuild2DMipmaps(GL_TEXTURE_2D, components, width, height, format, type, curData);

	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER, GL_LINEAR);

	//Sometimes you need to clamp images to the border to hide the seam
	if(clamp)
	{
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
	}

	//Got this far, must have worked
	if(!keepData){delete [] data; data = NULL;}

    return true;
}

void Image::UnLoad()
{
	if(glIsTexture(GL_ID))
	{
		glDeleteTextures(1, &GL_ID);
	}

	if(data!=NULL){delete [] data; data = NULL;}

	width = 0; height = 0; GL_ID = 0; 
	clamp = false; 
	keepData = false;
}

float Image::GetData(int U, int V, unsigned char component)
{
	if(data==NULL || component>components-1)return -1.0f;
	if(U<0 || U>width)return -1.0f;
	if(V<0 || V>height)return -1.0f;
	unsigned char out = data[U*components+component + V*width*components];
	return float(out)/255.0f;
}

float Image::GetData(float U, float V, unsigned char component)
{
	if(data==NULL || component>components-1)return -1.0f;
	if(U<0.0f || U>1.0f)return -1.0f;
	if(V<0.0f || V>1.0f)return -1.0f;
	unsigned char out = data[(int(U*float(width)+0.5f)*components+component)+int((1.0f-V)*float(height))*width*components];
	return float(out)/255.0f;
}

void Image::Activate()
{
	glEnable(GL_TEXTURE);
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, GL_ID);
}

void Image::SetClamp(bool clamp){this->clamp = clamp;}

void Image::SetKeepData(bool keep){keepData = keep;}

int Image::GetWidth()
{
	return width;
}

int Image::GetHeight()
{
	return height;
}
