//Brent Cowan - May. 19 / 2008

#include <fstream>         //Loading and saving files
using namespace std;

#include "GLFW/IncludesGLFW.h"
#include "Extensions.h"
#include "Texture.h" 
#include "Image.h"
#include "Shader.h"

Shader::Shader()
{
	shaderProgram = NULL;
}

Shader::~Shader()
{
	ext::glUseProgramObjectARB(0);
	ext::glDetachObjectARB(shaderProgram, glVertexShader);
	ext::glDetachObjectARB(shaderProgram, glPixelShader);
	ext::glDeleteObjectARB(shaderProgram);
}

bool Shader::Load(char *vertexShader, char *pixelShader, bool debug)
{
	int result;
	shaderProgram = ext::glCreateProgramObjectARB();

	//Load the vertex shader.
	char *ShaderCode = LoadShader(vertexShader);
	glVertexShader = ext::glCreateShaderObjectARB(GL_VERTEX_SHADER_ARB);
	ext::glShaderSourceARB(glVertexShader, 1, (const char**)&ShaderCode, NULL);
	ext::glCompileShaderARB(glVertexShader);
	ext::glGetObjectParameterivARB(glVertexShader, GL_OBJECT_COMPILE_STATUS_ARB, &result);
	ext::glAttachObjectARB(shaderProgram, glVertexShader);

	delete[] ShaderCode;
	ShaderCode = NULL;

	//Load the pixel shader.
	ShaderCode = LoadShader(pixelShader);
	glPixelShader = ext::glCreateShaderObjectARB(GL_FRAGMENT_SHADER_ARB);
	ext::glShaderSourceARB(glPixelShader, 1, (const char**)&ShaderCode, NULL);
	ext::glCompileShaderARB(glPixelShader);
	ext::glGetObjectParameterivARB(glPixelShader, GL_OBJECT_COMPILE_STATUS_ARB, &result);
	ext::glAttachObjectARB(shaderProgram, glPixelShader);

	delete[] ShaderCode;
	ShaderCode = NULL;

	// Link our shaders.
	ext::glLinkProgramARB(shaderProgram);
	ext::glGetObjectParameterivARB(shaderProgram, GL_OBJECT_LINK_STATUS_ARB, &result);

	if(debug)
	{
		char buff[9000];
		GLsizei len =9000;
		GLsizei outlen;
		//glGetInfoLogARB(shaderProgram, len, &outlen, buff);
		

		//Write the file to disk
		ofstream out;
		//create the file
		out.open("ShaderReport.html", ios::out | ios::binary);

		if(!out.fail()) //if we can write to the file
		{
			//html
			out << "<HTML><HEAD><TITLE>Shader Report</TITLE><BODY> ";

			//Output vertex shader debug
			out << "<H2>Vertex Shader File: " << vertexShader << "</H2>";
			ext::glGetInfoLogARB(glVertexShader, len, &outlen, buff);
			if(outlen != NULL)
			{
				out << "<FONT color=#ff0000>";
				for(int i=0; i<outlen; i++)
				{
					if(buff[i] == '\n')
					{
						out << "<BR>";
					}
					else
					out << buff[i];
				}
				out << "</FONT>";
			}
			else{out << endl << "(Compiled Successfully)" << "<BR>";}
			out << "<BR>";

			//Output pixel shader debug
			out << "<H2>Pixel Shader File:  " << pixelShader << "</H2>";
			ext::glGetInfoLogARB(glPixelShader, len, &outlen, buff);
			if(outlen != NULL)
			{
				out << "<FONT color=#ff0000>";
				for(int i=0; i<outlen; i++)
				{
					if(buff[i] == '\n')
					{
						out << "<BR>";
					}
					else
					out << buff[i];
				}
				out << "</FONT>";
			}
			else{out << "(Compiled Successfully)" << "<BR>";}
			out << "<BR>";

			//Output Link status
			out << "<H2>Linking Status:  " << "</H2>";
			ext::glGetInfoLogARB(shaderProgram, len, &outlen, buff);
			if(outlen != NULL)
			{
				for(int i=0; i<outlen; i++)
				{
					if(buff[i] == '\n')
					{
						out << "<BR>";
					}
					else
					out << buff[i];
				}
				out << "<BR>";
			}

			//ending html
			out << "</BODY></HTML> ";

			out.close();
		}
	}

	if(!result)return false;

	return true;
}

char* Shader::LoadShader(char *shaderName)
{
   char tempLine[512] = {0};
   int count = 0;

   // input object.
   ifstream tempInput;

   // Open the shader file.
   tempInput.open(shaderName);

   // If there is any errors then return false.
   if(!tempInput.is_open())
      {
         return NULL;
      }


   // Loop through each line of the file and get the total size.
   while(!tempInput.eof())
      {
         tempInput.getline(tempLine, 512, '\n');
         count++;
      }

   // Close the shader.
   tempInput.close();

   if(count == 0) return NULL;

   // Read in the data for use this time.
   ifstream input;

   // Create array to hold shader code.
   char *ShaderSource = new char[512 * count];

   // Re-open the shader and read in the whole thing into the array.
   input.open(shaderName);
   input.getline(ShaderSource , 512 * count, '\0');

   input.close(); // Close the shader.

   return ShaderSource;
}

void Shader::Activate()
{
	ext::glUseProgramObjectARB(shaderProgram);
}

void Shader::Deactivate()
{
	ext::glUseProgramObjectARB(0);

	glDisable(GL_BLEND);
	
	ext::glActiveTextureARB(GL_TEXTURE0_ARB); glDisable(GL_TEXTURE_2D);
	ext::glActiveTextureARB(GL_TEXTURE1_ARB); glDisable(GL_TEXTURE_2D);
	ext::glActiveTextureARB(GL_TEXTURE2_ARB); glDisable(GL_TEXTURE_2D);
	ext::glActiveTextureARB(GL_TEXTURE3_ARB); glDisable(GL_TEXTURE_2D);
	ext::glActiveTextureARB(GL_TEXTURE4_ARB); glDisable(GL_TEXTURE_2D);
	ext::glActiveTextureARB(GL_TEXTURE0_ARB);
}

void Shader::SendImage(Image* image, char* name, int imageNum)
{
	glEnable(GL_TEXTURE);
	ext::glActiveTextureARB(GL_TEXTURE0_ARB + imageNum); 
	glEnable(GL_TEXTURE_2D);
	image->Activate();
	GLuint textureID = ext::glGetUniformLocationARB(shaderProgram, name);
	ext::glUniform1iARB(textureID, imageNum);
	ext::glActiveTextureARB(GL_TEXTURE0_ARB); 
}

void Shader::SendFloat(char* name, float value)
{
	GLuint floatID = ext::glGetUniformLocationARB(shaderProgram, name);
	//if(floatID!=0)
	ext::glUniform1fARB(floatID, value);
}

void Shader::SendVec4(char* name, float x, float y, float z, float w)
{
	GLuint vecID = ext::glGetUniformLocationARB(shaderProgram, name);
	//if(vecID!=0)
	ext::glUniform4fARB(vecID, x, y, z, w);
}

void Shader::Blend(int RenderMode)
{
	glEnable(GL_BLEND);

	//Transparency Mode
	if(RenderMode == 0)      //Opaque  no transparency
	{
		glDisable(GL_ALPHA_TEST);
		glDisable(GL_BLEND);
	}
	else if(RenderMode == 1) //Threshold  all or nothing transparency
	{
		glEnable(GL_ALPHA_TEST);
		glAlphaFunc(GL_GREATER, 0.5f);
		glDisable(GL_BLEND);
	}
	else if(RenderMode == 2) //Blend  Normally
	{
		glDisable(GL_ALPHA_TEST);
		glEnable(GL_BLEND);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	}
	else if(RenderMode == 3)       //Blend - Additively
	{
		glDisable(GL_ALPHA_TEST);
		glEnable(GL_BLEND);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE);
	}
	else if(RenderMode == 4)       //Blend - Subtractive
	{
		glDisable(GL_ALPHA_TEST);
		glEnable(GL_BLEND);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE);
		ext::glBlendEquationEXT(GL_FUNC_REVERSE_SUBTRACT);//GL_FUNC_REVERSE_SUBTRACT
	}
	else      //Out of range! defualt to Opaque  no transparency
	{
		glDisable(GL_ALPHA_TEST);
		glDisable(GL_BLEND);
	}
}
