#include "stdafx.h"
#include "Vector2D.h"
#include <math.h>

float Vector2D::DTR = 3.14159f/180.0f; // PI / 180
float Vector2D::RTD = 180.0f/3.14159f; // 180 / PI

Vector2D::Vector2D()
{
   x = 0.0f;
   y = 0.0f;
}

Vector2D::Vector2D(float X, float Y)
{
   x = X;
   y = Y;
}


Vector2D::Vector2D(const Vector2D &v)
{
   x = v.x;
   y = v.y;
}

bool Vector2D::operator==(const Vector2D &v)
{
	if(x==v.x && y==v.y)return true; else return false;
}

void Vector2D::operator=(const Vector2D &v)
{
   x = v.x; y = v.y;
}


void Vector2D::operator+=(const Vector2D &v)
{
   x += v.x; y += v.y;
}


void Vector2D::operator-=(const Vector2D &v)
{
   x -= v.x; y -= v.y;
}


void Vector2D::operator/=(const Vector2D &v)
{
   x /= v.x; y /= v.y;
}


void Vector2D::operator*=(const Vector2D &v)
{
   x *= v.x; y *= v.y;
}


Vector2D Vector2D::operator+(const Vector2D &v2)
{
   return Vector2D(x + v2.x, y + v2.y);
}


Vector2D Vector2D::operator-(const Vector2D &v2)
{
   return Vector2D(x - v2.x, y - v2.y);
}


Vector2D Vector2D::operator/(const Vector2D &v2)
{
   return Vector2D(x / v2.x, y / v2.y);
}


Vector2D Vector2D::operator*(const Vector2D &v2)
{
   return Vector2D(x * v2.x, y * v2.y);
}

Vector2D Vector2D::operator/(float f)
{
   return Vector2D(x / f, y / f);
}


Vector2D Vector2D::operator*(float f)
{
   return Vector2D(x * f, y * f);
}

void Vector2D::Rotate(float angle)
{
	float rad = angle*DTR;
	float tempX = cos(rad)*x - sin(rad)*y;
	y = sin(rad)*x + cos(rad)*y;
	x = tempX;
}

float Vector2D::Dot(const Vector2D &v)
{
   return x * v.x + y * v.y;
}


float Vector2D::GetLength()
{
   return (float)sqrt(x * x + y * y);
}


void Vector2D::Normalize()
{
   float len = sqrt(x * x + y * y);

   if(len <= 0.00001f)
   {
	   x=1.0f; y=0.0f;
   }
   else
   {
		len = 1.0f / len;
		x *= len; y *= len;
   }
}

Vector2D Vector2D::AngleToVector(float angle)
{
	return Vector2D(cos(angle*DTR), sin(angle*DTR));
}

float Vector2D::VectorToAngle(const Vector2D &v)
{
	return atan2(v.y, v.x)*RTD;
}

Vector2D Vector2D::Perp()
{
  return Vector2D(-y, x);
}

float Vector2D::GetLengthSq()
{
  return (x * x + y * y);
}

void Vector2D::Reflect(Vector2D& norm)
{
  *this += (norm*-1.0f) * (this->Dot(norm)*2.0f);
}

void Vector2D::SetLength(float length)
{
	Normalize();
	x *= length;
	y *= length;
}
