//Brent Cowan May 7 2008
#include "stdafx.h"

#include <math.h>
#include <cstdlib>    // For srand() and rand()
#include "Vector3D.h"

float Vector3D::DTR = 3.14159f/180.0f;
float Vector3D::RTD = 57.295779f;      //(180/PI)
float Vector3D::sinAngle = 0.0f;
float Vector3D::cosAngle = 0.0f;
Vector3D Vector3D::tempV;

Vector3D::Vector3D()
{
   x = 0.0f;
   y = 0.0f;
   z = 0.0f;
}

Vector3D::Vector3D(float X, float Y, float Z)
{
   x = X;
   y = Y;
   z = Z;
}


Vector3D::Vector3D(const Vector3D &v)
{
   x = v.x;
   y = v.y;
   z = v.z;
}


void Vector3D::Add(const Vector3D &v1, const Vector3D &v2)
{
   x = v1.x + v2.x;
   y = v1.y + v2.y;
   z = v1.z + v2.z;
}


void Vector3D::Subtract(const Vector3D &v1, const Vector3D &v2)
{
   x = v1.x - v2.x;
   y = v1.y - v2.y;
   z = v1.z - v2.z;
}


void Vector3D::Multiply(const Vector3D &v1, const Vector3D &v2)
{
   x = v1.x * v2.x;
   y = v1.y * v2.y;
   z = v1.z * v2.z;
}


void Vector3D::Divide(const Vector3D &v1, const Vector3D &v2)
{
   x = v1.x / v2.x;
   y = v1.y / v2.y;
   z = v1.z / v2.z;
}


void Vector3D::Multiply(const Vector3D &v1, float f)
{
   x = v1.x * f;
   y = v1.y * f;
   z = v1.z * f;
}


void Vector3D::Divide(const Vector3D &v1, float f)
{
   if(f != 0.0f) f = 1.0f / f;

   x = v1.x * f;
   y = v1.y * f;
   z = v1.z * f;
}

bool Vector3D::operator==(const Vector3D &v)
{
	if(x==v.x && y==v.y && z==v.z)return true; else return false;
}

void Vector3D::operator=(const Vector3D &v)
{
   x = v.x;
   y = v.y;
   z = v.z;
}


void Vector3D::operator+=(const Vector3D &v)
{
   x += v.x;
   y += v.y;
   z += v.z;
}


void Vector3D::operator-=(const Vector3D &v)
{
   x -= v.x;
   y -= v.y;
   z -= v.z;
}


void Vector3D::operator/=(const Vector3D &v)
{
   x /= v.x;
   y /= v.y;
   z /= v.z;
}


void Vector3D::operator*=(const Vector3D &v)
{
   x *= v.x;
   y *= v.y;
   z *= v.z;
}


Vector3D Vector3D::operator+(const Vector3D &v2)
{
   return Vector3D(x + v2.x, y + v2.y, z + v2.z);
}


Vector3D Vector3D::operator-(const Vector3D &v2)
{
   return Vector3D(x - v2.x, y - v2.y, z - v2.z);
}


Vector3D Vector3D::operator/(const Vector3D &v2)
{
   return Vector3D(x / v2.x, y / v2.y, z / v2.z);
}


Vector3D Vector3D::operator*(const Vector3D &v2)
{
   return Vector3D(x * v2.x, y * v2.y, z * v2.z);
}

Vector3D Vector3D::operator/(float f)
{
   return Vector3D(x / f, y / f, z / f);
}


Vector3D Vector3D::operator*(float f)
{
   return Vector3D(x * f, y * f, z * f);
}

Vector3D Vector3D::Random()
{
	Vector3D temp;
	temp.x = float(rand()%1000)-500.0f;
	temp.y = float(rand()%1000)-500.0f;
	temp.z = float(rand()%1000)-500.0f;
	temp.Normalize();
	return temp;
}

void Vector3D::Negate()
{
   x = -x; y = -y; z = -z;
}


float Vector3D::Dot(const Vector3D &v)
{
   return x * v.x + y * v.y + z * v.z;
}


float Vector3D::GetLength()
{
   return (float)sqrt(x * x + y * y + z * z);
}


void Vector3D::Normalize()
{
   float len = sqrt(x * x + y * y + z * z);

   if(len <= 0.00001f) len = 1.0f;

   len = 1.0f / len;

   x *= len; y *= len; z *= len;
}


void Vector3D::Normalize(Vector3D p1, Vector3D p2, Vector3D p3)
{
   Vector3D e1 = p1 - p2;
   Vector3D e2 = p2 - p3;

   e1.Normalize();
   e2.Normalize();

   *this = e1.Cross(e2);
   Normalize();
}


Vector3D Vector3D::Cross(const Vector3D &v)
{
   return Vector3D(y * v.z - z * v.y,
                   z * v.x - x * v.z,
                   x * v.y - y * v.x);
}

//rotations
void Vector3D::RotateX(float angle)
{
	sinAngle = sin(DTR*angle);
	cosAngle = cos(DTR*angle);
	
	float tempY = y*cosAngle - z*sinAngle;
	z = y*sinAngle + z*cosAngle;
	y = tempY;
}

void Vector3D::RotateY(float angle)
{
	sinAngle = sin(DTR*angle);
	cosAngle = cos(DTR*angle);

	float tempX =  x*cosAngle + z*sinAngle;
	z = -x*sinAngle + z*cosAngle;
	x = tempX;
}

void Vector3D::RotateZ(float angle)
{
	sinAngle = sin(DTR*angle);
	cosAngle = cos(DTR*angle);
	
	float tempX = x*cosAngle - y*sinAngle;
	y = x*sinAngle + y*cosAngle;
	x = tempX;
}

void Vector3D::RotateAxis(float angle, const Vector3D &axis)
{
	Vector3D u = axis; u.Normalize();

	Vector3D rotMatrixRow0, rotMatrixRow1, rotMatrixRow2;

	sinAngle = sin(DTR*angle);
	cosAngle = cos(DTR*angle);
	float oneMinusCosAngle = 1.0f-cosAngle;

	rotMatrixRow0.x=(u.x)*(u.x) + cosAngle*(1-(u.x)*(u.x));
	rotMatrixRow0.y=(u.x)*(u.y)*(oneMinusCosAngle) - sinAngle*u.z;
	rotMatrixRow0.z=(u.x)*(u.z)*(oneMinusCosAngle) + sinAngle*u.y;

	rotMatrixRow1.x=(u.x)*(u.y)*(oneMinusCosAngle) + sinAngle*u.z;
	rotMatrixRow1.y=(u.y)*(u.y) + cosAngle*(1-(u.y)*(u.y));
	rotMatrixRow1.z=(u.y)*(u.z)*(oneMinusCosAngle) - sinAngle*u.x;
	
	rotMatrixRow2.x=(u.x)*(u.z)*(oneMinusCosAngle) - sinAngle*u.y;
	rotMatrixRow2.y=(u.y)*(u.z)*(oneMinusCosAngle) + sinAngle*u.x;
	rotMatrixRow2.z=(u.z)*(u.z) + cosAngle*(1-(u.z)*(u.z));

	x = Dot(rotMatrixRow0);
	y = Dot(rotMatrixRow1);
	z = Dot(rotMatrixRow2);
}

void Vector3D::polarCoord(float azemuth, float elevation)
{
	sinAngle = azemuth*DTR;
	cosAngle = elevation*DTR;

	tempV.x = sin(sinAngle);
	tempV.z = cos(sinAngle)*(-1.0f);
	tempV.y = cos(cosAngle);

	x = tempV.x * tempV.y;
	z = tempV.z * tempV.y;
	y = sin(cosAngle)*(-1.0f);
}

Vector3D Vector3D::vectorToPolar(Vector3D target)
{
	Vector3D output;

	output.y = 180.0f-(atan2(target.x, target.z)*RTD);

	if(output.y<0.0f){output.y+=360.0f;}

	tempV = target;
	tempV.y = 0.0f;
	output.x = (atan2(tempV.GetLength(), target.y)*RTD)-90.0f;

	if(output.x<0.0f){output.x+=360.0f;}

	output.z = (float)sqrt(target.x * target.x + target.y * target.y + target.z * target.z);

	return output;
}