
//Modified Brent Cowan Sept. 19 2007

#ifndef _BB_VECTOR_H_
#define _BB_VECTOR_H_

//#define DLL_EXPORT
#if defined DLL_EXPORT
#define DECLDIR __declspec(dllexport)
#else
#define DECLDIR __declspec(dllimport)
#endif

class DECLDIR Vector3D
{
private:
	static float DTR;
	static float RTD;
	static float sinAngle;
	static float cosAngle;
	static Vector3D tempV;
public:
	Vector3D();
	Vector3D(float X, float Y, float Z);
	Vector3D(const Vector3D &v);

	void Add(const Vector3D &v1, const Vector3D &v2);
	void Subtract(const Vector3D &v1, const Vector3D &v2);
	void Multiply(const Vector3D &v1, const Vector3D &v2);
	void Divide(const Vector3D &v1, const Vector3D &v2);

	void Multiply(const Vector3D &v1, float f);
	void Divide(const Vector3D &v1, float f);

	bool operator==(const Vector3D &v);
	void operator=(const Vector3D &v);
	void operator+=(const Vector3D &v);
	void operator-=(const Vector3D &v);
	void operator/=(const Vector3D &v);
	void operator*=(const Vector3D &v);

	Vector3D operator+(const Vector3D &v2);
	Vector3D operator-(const Vector3D &v2);      
	Vector3D operator/(const Vector3D &v2);
	Vector3D operator*(const Vector3D &v2);

	Vector3D operator/(float f);
	Vector3D operator*(float f);

	void Negate();
	Vector3D Random();
	float Dot(const Vector3D &v);
	float GetLength();

	void Normalize();
	void Normalize(Vector3D p1, Vector3D p2, Vector3D p3);
	Vector3D Cross(const Vector3D &v);

	//rotations
	void RotateX(float angle);
	void RotateY(float angle);
	void RotateZ(float angle);
	void RotateAxis(float angle, const Vector3D &axis);
	void polarCoord(float azemuth, float elevation);
	Vector3D vectorToPolar(Vector3D target);

	float x, y, z;
};

#endif