
#ifndef _WindowManager_
#define _WindowManager_

//#define DLL_EXPORT
#if defined DLL_EXPORT
#define DECLDIR __declspec(dllexport)
#else
#define DECLDIR __declspec(dllimport)
#endif

//define a container for color information
struct DECLDIR Color4f
{
	GLfloat R,G,B,A;
};

class DECLDIR WindowManager
{
private:
	unsigned short width;    // Window width (640, 800, 1024)
	unsigned short height;   // Window height (480, 600, 768)
	unsigned short x;        // Distance between left edge of window and left edge of screen
	unsigned short y;        // Distance between top of window and top of screen
	BYTE depthBufferBits;    // Size of depth buffer (16, 24, 32)
	bool fullScreen;	     // Full screen or windowed mode
	Color4f clearColor;		 // Color used when clearing the buffer
	bool clearColorBuff;     // Should the color buffer be cleared
	bool clearDepthBuff;     // Should the color buffer be cleared
	bool multisample;
	BYTE samples;
	BYTE textureQuality;
	string title;
	HWND hWnd;               // Window handle
	HDC hDC;
	float aspectRatio;

public:
	WindowManager();
	bool Open();
	bool IsOpen();
	void Close();

	void SetSize(int width, int height);
	void SetPosition(int x, int y);
	void SetTitle(const char* title);
	void SetDepthBufferBits(int bits);
	void SetFullScreen(bool fullScreen);
	void SetClearColor(float r, float g, float b, float a);
	void SetClearColor(float r, float g, float b);
	void SetClearColorBuffer(bool clear);
	void SetClearDepthBuffer(bool clear);
	void SetSampleRate(int rate);
	void SetTextureQuality(int Quality);
	void SetVerticalSync(bool vSync);

	bool    GetClearColorBuffer();
	bool    GetClearDepthBuffer();
	float   GetAspectRatio();
	Color4f GetClearColor();
	int     GetHeight();
	int     GetWidth();
	HDC     GetHDC();
};

#endif	