#include "Gadget/Engine.h" //This file includes all the other engine files

int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
	//Here I'll create an instance of "Core" and call it "G". You can of course name it anything you like, 
	//but keeping the name short is important because we will end up typing it a lot. If you type G followed by 
	//a period, you will see a drop down list of all the stuff the engine can do, well not all of it. There are 
	//several other support classes provided to load graphics, play sounds, and other stuff.
	Core G; //Create an instance of the game engine
	G.Window.SetSampleRate(4); //Renders the text with smoother edges
	G.Window.SetClearColor(0.2f, 0.4f, 0.5f); //Set the background color to green
	G.Window.Open(); //This line of code creates a window.

	/*
	FMOD is a commercial sound engine used in many of the games you play. Its a full featured sound engine, but to make 
	things simple I made a wrapper class that reduces its functionality somewhat. I have kept it separate from the rest 
	of the code in case I want to replace it with another sound engine later.
	*/
	SoundEngine S; //Create an instance of the sound engine, Only make one!

	Sound drumSnd; //Create a sound
	//The load function needs the file name of the sound and the address of the sound engine
	drumSnd.Load("Sounds/drumloop.wav", &S, false, true); //load the sound and set it to loop mode
	drumSnd.Play(); //Start playing the sound

	//Make sure the window is open before you load any fonts!
	Font timesFnt;
	timesFnt.Load(G.Window.GetHDC(), "Times New Roman"); //Load a font

	TextBox messageTxt;
	messageTxt.Init(&G, &timesFnt, 700.0f, 400.0f);//Set the fond and size of the text box
	messageTxt.position = Vector2D(50.0f, 450.0f); //Place it somewhere
	messageTxt.SetScale(85.0f); //Set the size of the text (each row of text will be roughly 85 pixels high)
	messageTxt.AlignCenter(); //Paragraph alignment
	messageTxt.SetText("If you can hear the drums, than the engine is working properly!");
	
	
	while(G.Window.IsOpen()) //Loop continuously as long as the engine is running
	{
		G.Renderer.BeginFrame(); //Clears the buffers
		G.Renderer.Render2D();   //Render a 2D scene

			/*
			You can check to see if a key is being pressed at any time using the standard key codes or by using characters. For 
			example, I could check to see if the letter B is pressed like this G.Keyboard.GetKeyPress(B).
			*/
			if(G.Keyboard.GetKeyHold(GLFW_KEY_ESC)) {G.Window.Close();}	//ESC key closes the window
			
			//Shader effects and blending will affect everything you render! So it's a good idea to turn them off 
			//when you're finished with them. In fact, some older video cards will run really slow if shaders are
			//active when the screen refreshes. It's a good idea to turn them off when you are finished rendering
			//your scene. Also do this before rendering text!
			Shader::Deactivate(); //Turns off blending and any active shader

			glColor3f(0.8f, 1.0f, 1.0f);//I want the text to be light blue
			messageTxt.Render(); //Render the text
			glColor3f(1.0f, 1.0f, 1.0f);//Always set the color back to white or it will color everything!

			S.Update(G.Timer.GetElapsedTime()); //The sound engine has to be updated continuously or the sounds stop playing!

		G.Renderer.EndFrame(); //Finished rendering, display the result
	}
	return 0;
}			